import { getAssembliesDir } from "fable-metadata";
import { getVersion as getVersion_1, getFableLibDir as getFableLibDir_1 } from "../src/util.js";
import { init } from "fable-standalone";
import { coreAssemblies } from "./Fable.Transforms/Global/Metadata.js";
import { toText, join, substring, toFail, printf, toConsole, endsWith, replace, trimEnd } from "./fable_modules/fable-library.4.5.0/String.js";
import { stringHash, uncurry2, defaultOf, comparePrimitives, Lazy, int32ToString } from "./fable_modules/fable-library.4.5.0/Util.js";
import { runCmdAndExitIfFails, CmdLineOptions, Path_GetFileNameWithoutExtension, serializeToJson, ensureDirExists, writeAllText, readAllBytes as readAllBytes_1, measureTime, getHomePath, readAllText, Path_ChangeExtension, normalizeFullPath, Path_GetFileName, getRelativePath, Path_Combine, Path_GetDirectoryName, normalizePath } from "./Platform.js";
import { ofSeq as ofSeq_1, FSharpSet__Contains } from "./fable_modules/fable-library.4.5.0/Set.js";
import { singleton } from "./fable_modules/fable-library.4.5.0/AsyncBuilder.js";
import { StringBuilder_$ctor, StringBuilder__Append_Z721C83C5 } from "./fable_modules/fable-library.4.5.0/System.Text.js";
import { MappingIndex } from "./fable_modules/source-map-sharp.1.0.8/src/Util.fs.js";
import { SourceMapGenerator__toJSON, SourceMapGenerator_$ctor_Z257FAB7, SourceMapGenerator__AddMapping_Z1908099D } from "./fable_modules/source-map-sharp.1.0.8/src/SourceMapGenerator.fs.js";
import { toArray as toArray_1, orElse, map as map_1, defaultArg, unwrap } from "./fable_modules/fable-library.4.5.0/Option.js";
import { class_type } from "./fable_modules/fable-library.4.5.0/Reflection.js";
import { FSharpRef, toString } from "./fable_modules/fable-library.4.5.0/Types.js";
import { equalsWith, splitAt, tryFindIndex, contains as contains_1, pairwise, tryFindBack, append, reverse, map, partition } from "./fable_modules/fable-library.4.5.0/Array.js";
import { startImmediate } from "./fable_modules/fable-library.4.5.0/Async.js";
import { parseProject } from "./ProjectParser.js";
import { tryFind, toArray, ofSeq } from "./fable_modules/fable-library.4.5.0/Map.js";
import { addToDict, tryGetValue } from "./fable_modules/fable-library.4.5.0/MapUtil.js";
import { ofArray, singleton as singleton_1, contains } from "./fable_modules/fable-library.4.5.0/List.js";
import { tryParse } from "./fable_modules/fable-library.4.5.0/Boolean.js";
import { iterate } from "./fable_modules/fable-library.4.5.0/Seq.js";

export const getMetadataDir = getAssembliesDir;

export const getFableLibDir = getFableLibDir_1;

export const getVersion = getVersion_1;

export const initFable = init;

export const references = coreAssemblies;

export const metadataPath = trimEnd(getMetadataDir(), "\\", "/") + "/";

export function Imports_trimPath(path) {
    return replace(replace(replace(path, "../", ""), "./", ""), ":", "");
}

export function Imports_isRelativePath(path) {
    if (path.indexOf("./") === 0) {
        return true;
    }
    else {
        return path.indexOf("../") === 0;
    }
}

export function Imports_isAbsolutePath(path) {
    if (path.indexOf("/") === 0) {
        return true;
    }
    else {
        return path.indexOf(":") === 1;
    }
}

export function Imports_preventConflicts(conflicts, originalName) {
    const check = (originalName_1_mut, n_mut) => {
        check:
        while (true) {
            const originalName_1 = originalName_1_mut, n = n_mut;
            const name = (n > 0) ? ((originalName_1 + "_") + int32ToString(n)) : originalName_1;
            if (!conflicts(name)) {
                return name;
            }
            else {
                originalName_1_mut = originalName_1;
                n_mut = (n + 1);
                continue check;
            }
            break;
        }
    };
    return check(originalName, 0);
}

export function Imports_getTargetAbsolutePath(getOrAddDeduplicateTargetDir, importPath, projDir, outDir) {
    const importPath_1 = normalizePath(importPath);
    const outDir_1 = normalizePath(outDir);
    if (importPath_1.indexOf(outDir_1 + "/") === 0) {
        return importPath_1;
    }
    else {
        const importDir = Path_GetDirectoryName(importPath_1);
        return Path_Combine(getOrAddDeduplicateTargetDir(importDir, (currentTargetDirs) => Imports_preventConflicts((value) => FSharpSet__Contains(currentTargetDirs, value), Path_Combine(outDir_1, Imports_trimPath(getRelativePath(projDir, importDir))))), Path_GetFileName(importPath_1));
    }
}

export function Imports_getTargetRelativePath(getOrAddDeduplicateTargetDir, importPath, targetDir, projDir, outDir) {
    const relPath = getRelativePath(targetDir, Imports_getTargetAbsolutePath(getOrAddDeduplicateTargetDir, importPath, projDir, outDir));
    if (Imports_isRelativePath(relPath)) {
        return relPath;
    }
    else {
        return "./" + relPath;
    }
}

export function Imports_getImportPath(getOrAddDeduplicateTargetDir, sourcePath, targetPath, projDir, outDir, importPath) {
    if (outDir != null) {
        const outDir_1 = outDir;
        const importPath_1 = (importPath.indexOf("${outDir}") === 0) ? normalizeFullPath(Path_Combine(outDir_1, replace(importPath, "${outDir}", ""))) : importPath;
        const sourceDir = Path_GetDirectoryName(sourcePath);
        const targetDir = Path_GetDirectoryName(targetPath);
        const importPath_2 = Imports_isRelativePath(importPath_1) ? normalizeFullPath(Path_Combine(sourceDir, importPath_1)) : importPath_1;
        if (Imports_isAbsolutePath(importPath_2)) {
            if (endsWith(importPath_2, ".fs")) {
                return Imports_getTargetRelativePath(getOrAddDeduplicateTargetDir, importPath_2, targetDir, projDir, outDir_1);
            }
            else {
                return getRelativePath(targetDir, importPath_2);
            }
        }
        else {
            return importPath_2;
        }
    }
    else {
        return replace(importPath, "${outDir}", ".");
    }
}

export class SourceWriter {
    constructor(sourcePath, targetPath, projDir, options, fileExt, dedupTargetDir) {
        this.sourcePath = sourcePath;
        this.targetPath = targetPath;
        this.projDir = projDir;
        this.options = options;
        this.dedupTargetDir = dedupTargetDir;
        this["fileExt@86"] = (endsWith(fileExt, ".ts") ? Path_ChangeExtension(fileExt, ".js") : fileExt);
        this.sb = StringBuilder_$ctor();
        this.mapGenerator = (new Lazy(SourceMapGenerator_$ctor_Z257FAB7));
    }
    Write(str) {
        const _ = this;
        return singleton.Delay(() => singleton.Return(void StringBuilder__Append_Z721C83C5(_.sb, str)));
    }
    MakeImportPath(path) {
        const _ = this;
        const path_1 = Imports_getImportPath(_.dedupTargetDir, _.sourcePath, _.targetPath, _.projDir, _.options.outDir, path);
        return endsWith(path_1, ".fs") ? Path_ChangeExtension(path_1, _["fileExt@86"]) : path_1;
    }
    AddSourceMapping(_arg) {
        const _ = this;
        if (_.options.sourceMaps) {
            const generated = new MappingIndex(_arg[2], _arg[3]);
            const original = new MappingIndex(_arg[0], _arg[1]);
            SourceMapGenerator__AddMapping_Z1908099D(_.mapGenerator.Value, generated, original, _.sourcePath, unwrap(_arg[4]));
        }
    }
    Dispose() {
    }
}

export function SourceWriter_$reflection() {
    return class_type("Fable.Compiler.App.SourceWriter", void 0, SourceWriter);
}

export function SourceWriter_$ctor_71349DF3(sourcePath, targetPath, projDir, options, fileExt, dedupTargetDir) {
    return new SourceWriter(sourcePath, targetPath, projDir, options, fileExt, dedupTargetDir);
}

export function SourceWriter__get_SourceMap(_) {
    return SourceMapGenerator__toJSON(_.mapGenerator.Value);
}

export function SourceWriter__get_Result(_) {
    return toString(_.sb);
}

export function printErrors(showWarnings, errors) {
    const printError = (e) => {
        const errorType = e.IsWarning ? "Warning" : "Error";
        toConsole(printf("%s"))(`${e.FileName} (${e.StartLine},${e.StartColumn}): ${errorType}: ${e.Message}`);
    };
    const patternInput = partition((e_1) => e_1.IsWarning, errors);
    const errors_1 = patternInput[1];
    const hasErrors = !(errors_1.length === 0);
    if (showWarnings) {
        patternInput[0].forEach(printError);
    }
    if (hasErrors) {
        errors_1.forEach(printError);
        throw new Error("Too many errors.");
    }
}

export function runAsync(computation) {
    startImmediate(singleton.Delay(() => singleton.TryWith(singleton.Delay(() => singleton.Bind(computation, () => singleton.Return(void 0))), (_arg_1) => {
        const e = _arg_1;
        const arg = e.message;
        toConsole(printf("[ERROR] %s"))(arg);
        const arg_1 = e.stack;
        toConsole(printf("%s"))(arg_1);
        return singleton.Zero();
    })));
}

export function parseFiles(projectFileName, options) {
    const patternInput = parseProject(projectFileName);
    const fileNames = patternInput[1];
    const sources = map(readAllText, fileNames);
    const nugetPath = Path_Combine(replace(getHomePath(), "\\", "/"), ".nuget");
    const fileNames_1 = map((x) => replace(x, nugetPath, ""), fileNames);
    const dllRefMap = ofSeq(map((x_1) => [Path_GetFileName(x_1), x_1], reverse(patternInput[0])), {
        Compare: comparePrimitives,
    });
    const references_1 = append(references, map((tuple) => tuple[0], toArray(dllRefMap)));
    const fable = initFable();
    const otherOptions_1 = append(["--optimize" + (options.optimize ? "+" : "-")], patternInput[2]);
    const patternInput_1 = measureTime(() => fable.CreateChecker(references_1, (dllName_1) => {
        let dllName, value;
        return readAllBytes_1((dllName = dllName_1, (value = (metadataPath + dllName), defaultArg(tryFind(dllName, dllRefMap), value))));
    }, otherOptions_1), void 0);
    const checker = patternInput_1[0];
    const arg = getVersion();
    toConsole(printf("fable-compiler-js v%s"))(arg);
    toConsole(printf("--------------------------------------------"));
    toConsole(printf("InteractiveChecker created in %d ms"))(patternInput_1[1]);
    const patternInput_2 = measureTime(() => fable.ParseAndCheckProject(checker, projectFileName, fileNames_1, sources), void 0);
    const parseRes = patternInput_2[0];
    toConsole(printf("Project: %s, FCS time: %d ms"))(projectFileName)(patternInput_2[1]);
    toConsole(printf("--------------------------------------------"));
    const showWarnings = !options.benchmark;
    printErrors(showWarnings, parseRes.Errors);
    if (options.benchmark) {
    }
    else {
        fable.ClearCache(checker);
        const fileNames_2 = fileNames_1.filter((x_3) => !endsWith(x_3, ".fsi"));
        const projDir = Path_GetDirectoryName(normalizeFullPath(projectFileName));
        const libDir = normalizeFullPath(defaultArg(options.libDir, getFableLibDir()));
        let fileExt;
        const matchValue = options.language.toLowerCase();
        switch (matchValue) {
            case "js":
            case "javascript": {
                fileExt = ".js";
                break;
            }
            case "ts":
            case "typescript": {
                fileExt = ".ts";
                break;
            }
            case "py":
            case "python": {
                fileExt = ".py";
                break;
            }
            case "php": {
                fileExt = ".php";
                break;
            }
            case "dart": {
                fileExt = ".dart";
                break;
            }
            case "rust": {
                fileExt = ".rs";
                break;
            }
            default:
                fileExt = toFail(printf("Unsupported language: %s"))(options.language);
        }
        const fileExt_1 = (options.outDir == null) ? (".fs" + fileExt) : fileExt;
        let getOrAddDeduplicateTargetDir;
        const dedupDic = new Map([]);
        getOrAddDeduplicateTargetDir = ((importDir) => ((addTargetDir) => {
            const importDir_1 = normalizeFullPath(importDir).toLocaleLowerCase();
            let matchValue_1;
            let outArg = defaultOf();
            matchValue_1 = [tryGetValue(dedupDic, importDir_1, new FSharpRef(() => outArg, (v) => {
                outArg = v;
            })), outArg];
            if (matchValue_1[0]) {
                return matchValue_1[1];
            }
            else {
                const v_2 = addTargetDir(ofSeq_1(dedupDic.values(), {
                    Compare: comparePrimitives,
                }));
                addToDict(dedupDic, importDir_1, v_2);
                return v_2;
            }
        }));
        runAsync(singleton.Delay(() => singleton.For(fileNames_2, (_arg) => {
            let fsAstStr;
            const fileName_1 = _arg;
            const patternInput_3 = measureTime((tupledArg) => fable.CompileToTargetAst(libDir, tupledArg[0], tupledArg[1], options.typedArrays, options.language), [parseRes, fileName_1]);
            const res_1 = patternInput_3[0];
            toConsole(printf("File: %s, Fable time: %d ms"))(fileName_1)(patternInput_3[1]);
            printErrors(showWarnings, res_1.FableErrors);
            let outPath;
            const matchValue_2 = options.outDir;
            outPath = ((matchValue_2 != null) ? Path_ChangeExtension(Imports_getTargetAbsolutePath(uncurry2(getOrAddDeduplicateTargetDir), fileName_1, projDir, matchValue_2), fileExt_1) : Path_ChangeExtension(fileName_1, fileExt_1));
            return singleton.Combine(options.printAst ? ((fsAstStr = fable.FSharpAstToString(parseRes, fileName_1), (writeAllText(substring(outPath, 0, outPath.lastIndexOf(fileExt_1)) + ".fs.ast", fsAstStr), singleton.Zero()))) : singleton.Zero(), singleton.Delay(() => {
                const writer = SourceWriter_$ctor_71349DF3(fileName_1, outPath, projDir, options, fileExt_1, uncurry2(getOrAddDeduplicateTargetDir));
                return singleton.Bind(fable.PrintTargetAst(res_1, writer), () => {
                    let mapPath, sourceMapUrl;
                    ensureDirExists(Path_GetDirectoryName(outPath));
                    return singleton.Combine(options.sourceMaps ? ((mapPath = (outPath + ".map"), (sourceMapUrl = ("//# sourceMappingURL=" + Path_GetFileName(mapPath)), singleton.Bind(writer.Write(sourceMapUrl), () => {
                        writeAllText(mapPath, serializeToJson(SourceWriter__get_SourceMap(writer)));
                        return singleton.Zero();
                    })))) : singleton.Zero(), singleton.Delay(() => {
                        writeAllText(outPath, SourceWriter__get_Result(writer));
                        return singleton.Zero();
                    }));
                });
            }));
        })));
    }
}

export function argValue(keys, args) {
    return map_1((tuple) => tuple[1], tryFindBack((tupledArg) => {
        if (!(tupledArg[1].indexOf("-") === 0)) {
            return contains(tupledArg[0], keys, {
                Equals: (x, y) => (x === y),
                GetHashCode: stringHash,
            });
        }
        else {
            return false;
        }
    }, pairwise(args)));
}

export function tryFlag(flag, args) {
    const matchValue = argValue(singleton_1(flag), args);
    if (matchValue == null) {
        if (contains_1(flag, args, {
            Equals: (x, y) => (x === y),
            GetHashCode: stringHash,
        })) {
            return true;
        }
        else {
            return void 0;
        }
    }
    else {
        let matchValue_1;
        let outArg = false;
        matchValue_1 = [tryParse(matchValue, new FSharpRef(() => outArg, (v) => {
            outArg = v;
        })), outArg];
        if (matchValue_1[0]) {
            return matchValue_1[1];
        }
        else {
            return void 0;
        }
    }
}

export function hasFlag(flag, args) {
    return defaultArg(tryFlag(flag, args), false);
}

export function run(opts, projectFileName, outDir) {
    const commandToRun = map_1((i) => {
        const scriptFile = Path_Combine(defaultArg(outDir, "."), Path_GetFileNameWithoutExtension(projectFileName) + ".js");
        const runArgs = join(" ", opts.slice(i + 1, opts.length));
        return toText(printf("node %s %s"))(scriptFile)(runArgs);
    }, tryFindIndex((y) => ("--run" === y), opts));
    parseFiles(projectFileName, new CmdLineOptions(orElse(argValue(ofArray(["--outDir", "-o"]), opts), outDir), argValue(singleton_1("--fableLib"), opts), hasFlag("--benchmark", opts), hasFlag("--optimize", opts), hasFlag("--sourceMaps", opts) ? true : hasFlag("-s", opts), tryFlag("--typedArrays", opts), defaultArg(map_1((_arg) => "TypeScript", argValue(ofArray(["--language", "--lang"]), opts)), "JavaScript"), hasFlag("--printAst", opts)));
    iterate((cmd) => {
        runCmdAndExitIfFails(cmd);
    }, toArray_1(commandToRun));
}

export function parseArguments(argv) {
    let testExpr, testExpr_1;
    const usage = "Usage: fable <F#_PROJECT_PATH> [OUT_DIR] [--options]\n\nOptions:\n  --help            Show help\n  --version         Print version\n  -o|--outDir       Redirect compilation output to a directory\n  -s|--sourceMaps   Enable source maps\n  --fableLib        Specify Fable library path\n  --typedArrays     Compile numeric arrays as JS typed arrays (default is true for JS, false for TS)\n  --run             Execute the script after compilation\n\n  --optimize        Compile with optimized F# AST (experimental)\n";
    let patternInput;
    const matchValue = tryFindIndex((s) => (s.indexOf("-") === 0), argv);
    patternInput = ((matchValue != null) ? splitAt(matchValue, argv) : [argv, []]);
    const opts = patternInput[1];
    const args = patternInput[0];
    if (hasFlag("--help", argv)) {
        toConsole(printf("%s"))(usage);
    }
    else if (hasFlag("--version", argv)) {
        const arg_1 = getVersion();
        toConsole(printf("v%s"))(arg_1);
    }
    else if ((testExpr = args, !equalsWith((x, y) => (x === y), testExpr, defaultOf()) && (testExpr.length === 1))) {
        run(opts, args[0], void 0);
    }
    else if ((testExpr_1 = args, !equalsWith((x_1, y_1) => (x_1 === y_1), testExpr_1, defaultOf()) && (testExpr_1.length === 2))) {
        run(opts, args[0], args[1]);
    }
    else {
        toConsole(printf("%s"))(usage);
    }
}

(function (argv) {
    try {
        parseArguments(argv);
    }
    catch (ex) {
        const arg = ex.message;
        const arg_1 = ex.stack;
        toConsole(printf("Error: %s\n%s"))(arg)(arg_1);
    }
    return 0;
})(typeof process === 'object' ? process.argv.slice(2) : []);

