import { bind, toArray, unwrap, value as value_8, defaultArg } from "../../fable-library.4.5.0/Option.js";
import { ArraySet$1__indexOf_2B595, ArraySet$1__ToArray, ArraySet$1__Add_Z1FE5A521, ArraySet$1__Has_2B595, ArraySet$1_$ctor } from "./ArraySet.fs.js";
import { MappingList__ToArray, MappingList__Add_Z3411214C, MappingList_$ctor } from "./MappingList.fs.js";
import { class_type } from "../../fable-library.4.5.0/Reflection.js";
import { printf, toText } from "../../fable-library.4.5.0/String.js";
import { compareByGeneratedPositionsInflated, SourceGeneratorJSON, getRelativePath, Mapping } from "./Util.fs.js";
import { iterate, toList, length, map } from "../../fable-library.4.5.0/Seq.js";
import { getItemFromDict } from "../../fable-library.4.5.0/MapUtil.js";
import { StringBuilder__Append_Z721C83C5, StringBuilder_$ctor } from "../../fable-library.4.5.0/System.Text.js";
import { disposeSafe, getEnumerator } from "../../fable-library.4.5.0/Util.js";
import { rangeDouble } from "../../fable-library.4.5.0/Range.js";
import { Base64Vlq_Encode_Z524259A4 } from "./Base64VLQ.fs.js";
import { toString } from "../../fable-library.4.5.0/Types.js";

export class SourceMapGenerator {
    constructor(skipValidation, file, sourceRoot) {
        this._file = file;
        this._sourceRoot = sourceRoot;
        this._skipValidation = defaultArg(skipValidation, false);
        this._names = ArraySet$1_$ctor();
        this._mappings = MappingList_$ctor();
        this["_sourcesContents@"] = (new Map([]));
        this["_sources@"] = ArraySet$1_$ctor();
    }
}

export function SourceMapGenerator_$reflection() {
    return class_type("SourceMapSharp.SourceMapGenerator", void 0, SourceMapGenerator);
}

export function SourceMapGenerator_$ctor_Z257FAB7(skipValidation, file, sourceRoot) {
    return new SourceMapGenerator(skipValidation, file, sourceRoot);
}

export function SourceMapGenerator__get__sourcesContents(__) {
    return __["_sourcesContents@"];
}

export function SourceMapGenerator__get__sources(__) {
    return __["_sources@"];
}

export function SourceMapGenerator_get_version() {
    return 3;
}

export function SourceMapGenerator_ValidateMapping_Z1908099D(generated, original, source, name) {
    let arg;
    let isInvalid = true;
    if (((((generated.line > 0) && (generated.column >= 0)) && (original == null)) && (source == null)) && (name == null)) {
        isInvalid = false;
    }
    else if ((((((generated.line > 0) && (generated.column >= 0)) && (original != null)) && (value_8(original).line > 0)) && (value_8(original).column >= 0)) && (source != null)) {
        isInvalid = false;
    }
    if (isInvalid) {
        throw new Error((arg = {
            generated: generated,
            name: unwrap(name),
            original: unwrap(original),
            source: unwrap(source),
        }, toText(printf("Invalid mapping: %A"))(arg)));
    }
}

export function SourceMapGenerator__AddMapping_Z1908099D(this$, generated, original, source, name) {
    if (!this$._skipValidation) {
        SourceMapGenerator_ValidateMapping_Z1908099D(generated, original, source, name);
    }
    if (source != null) {
        if (!ArraySet$1__Has_2B595(SourceMapGenerator__get__sources(this$), value_8(source))) {
            ArraySet$1__Add_Z1FE5A521(SourceMapGenerator__get__sources(this$), value_8(source), false);
        }
    }
    if (name != null) {
        if (!ArraySet$1__Has_2B595(this$._names, value_8(name))) {
            ArraySet$1__Add_Z1FE5A521(this$._names, value_8(name), false);
        }
    }
    MappingList__Add_Z3411214C(this$._mappings, new Mapping(generated, original, source, name));
}

export function SourceMapGenerator__GenerateSourcesContent_67C6C06C(this$, aSources, aSourceRoot) {
    return map((source_1) => {
        if (length(SourceMapGenerator__get__sourcesContents(this$).keys()) === 0) {
            return void 0;
        }
        else {
            let s = source_1;
            if (aSourceRoot != null) {
                s = getRelativePath(value_8(this$._sourceRoot), source_1);
            }
            if (SourceMapGenerator__get__sourcesContents(this$).has(s)) {
                return getItemFromDict(SourceMapGenerator__get__sourcesContents(this$), s);
            }
            else {
                return void 0;
            }
        }
    }, Array.from(aSources));
}

export function SourceMapGenerator__SetSourceContent_27AED5E3(this$, aSourceFile, aSourceContent) {
    let source = aSourceFile;
    if (this$._sourceRoot != null) {
        source = getRelativePath(value_8(this$._sourceRoot), source);
    }
    if (aSourceContent != null) {
        SourceMapGenerator__get__sourcesContents(this$).set(source, value_8(aSourceContent));
    }
    else if (length(SourceMapGenerator__get__sourcesContents(this$).keys()) > 0) {
        SourceMapGenerator__get__sourcesContents(this$).delete(source);
    }
}

export function SourceMapGenerator__toJSON(this$) {
    const matchValue = SourceMapGenerator_get_version() | 0;
    const sources = ArraySet$1__ToArray(SourceMapGenerator__get__sources(this$));
    return new SourceGeneratorJSON(matchValue, sources, ArraySet$1__ToArray(this$._names), SourceMapGenerator__SerializeMappings(this$), this$._file, (length(SourceMapGenerator__get__sourcesContents(this$).keys()) > 0) ? SourceMapGenerator__GenerateSourcesContent_67C6C06C(this$, sources, this$._sourceRoot) : void 0, this$._sourceRoot);
}

export function SourceMapGenerator__SerializeMappings(this$) {
    let objectArg;
    let previousGeneratedColumn = 0;
    let previousGeneratedLine = 1;
    let previousOriginalColumn = 0;
    let previousOriginalLine = 0;
    let previousName = 0;
    let previousSource = 0;
    const result = StringBuilder_$ctor();
    let next = StringBuilder_$ctor();
    let nameIdx = 0;
    let sourceIdx = 0;
    const mappings = MappingList__ToArray(this$._mappings);
    const enumerator = getEnumerator(toList(rangeDouble(0, 1, mappings.length - 1)));
    try {
        while (enumerator["System.Collections.IEnumerator.MoveNext"]()) {
            const i = enumerator["System.Collections.Generic.IEnumerator`1.get_Current"]() | 0;
            let shouldContinue = false;
            const mapping = mappings[i];
            next = StringBuilder_$ctor();
            if (mapping.Generated.line !== previousGeneratedLine) {
                previousGeneratedColumn = 0;
                while (mapping.Generated.line !== previousGeneratedLine) {
                    StringBuilder__Append_Z721C83C5(next, ";");
                    previousGeneratedLine = ((previousGeneratedLine + 1) | 0);
                }
            }
            else if (i > 0) {
                if (compareByGeneratedPositionsInflated(mapping, mappings[i - 1]) === 0) {
                    shouldContinue = true;
                }
                else {
                    StringBuilder__Append_Z721C83C5(next, ",");
                }
            }
            if (!shouldContinue) {
                StringBuilder__Append_Z721C83C5(next, Base64Vlq_Encode_Z524259A4(mapping.Generated.column - previousGeneratedColumn));
                previousGeneratedColumn = (mapping.Generated.column | 0);
                if (mapping.Source != null) {
                    iterate((indexOfMappingSource) => {
                        sourceIdx = (indexOfMappingSource | 0);
                        StringBuilder__Append_Z721C83C5(next, Base64Vlq_Encode_Z524259A4(sourceIdx - previousSource));
                        previousSource = (sourceIdx | 0);
                    }, toArray(bind((objectArg = SourceMapGenerator__get__sources(this$), (aStr) => ArraySet$1__indexOf_2B595(objectArg, aStr)), mapping.Source)));
                    iterate((original) => {
                        StringBuilder__Append_Z721C83C5(next, Base64Vlq_Encode_Z524259A4((original.line - 1) - previousOriginalLine));
                        previousOriginalLine = ((original.line - 1) | 0);
                        StringBuilder__Append_Z721C83C5(next, Base64Vlq_Encode_Z524259A4(original.column - previousOriginalColumn));
                        previousOriginalColumn = (original.column | 0);
                    }, toArray(mapping.Original));
                    if (mapping.Name != null) {
                        iterate((indexOfMappingName) => {
                            nameIdx = (indexOfMappingName | 0);
                            StringBuilder__Append_Z721C83C5(next, Base64Vlq_Encode_Z524259A4(nameIdx - previousName));
                            previousName = (nameIdx | 0);
                        }, toArray(bind((aStr_1) => ArraySet$1__indexOf_2B595(this$._names, aStr_1), mapping.Name)));
                    }
                }
                StringBuilder__Append_Z721C83C5(result, toString(next));
            }
        }
    }
    finally {
        disposeSafe(enumerator);
    }
    return toString(result);
}

