import { FSharpRef, toString, Union, Record } from "../../fable-library.4.5.0/Types.js";
import { class_type, union_type, array_type, record_type, string_type, option_type, int32_type } from "../../fable-library.4.5.0/Reflection.js";
import { StringBuilder__Append_Z721C83C5, StringBuilder_$ctor } from "../../fable-library.4.5.0/System.Text.js";
import { equals, disposeSafe, getEnumerator, defaultOf } from "../../fable-library.4.5.0/Util.js";
import { map as map_1, insertRangeInPlace } from "../../fable-library.4.5.0/Array.js";
import { toList, iterate, tryLast } from "../../fable-library.4.5.0/Seq.js";
import { replace } from "../../fable-library.4.5.0/String.js";
import { SourceMapGenerator__SetSourceContent_27AED5E3, SourceMapGenerator__AddMapping_Z1908099D, SourceMapGenerator_$ctor_Z257FAB7 } from "./SourceMapGenerator.fs.js";
import { MappingIndex } from "./Util.fs.js";
import { value as value_1 } from "../../fable-library.4.5.0/Option.js";
import { rangeDouble } from "../../fable-library.4.5.0/Range.js";

export const SourceNodeModule_REGEX_NEWLINE = /(\r?\n)/gu;

export const SourceNodeModule_NEWLINE_CODE = 10;

export class SourceMappingOriginal extends Record {
    constructor(line, column, Source, Name) {
        super();
        this.line = line;
        this.column = column;
        this.Source = Source;
        this.Name = Name;
    }
}

export function SourceMappingOriginal_$reflection() {
    return record_type("SourceMapSharp.SourceMappingOriginal", [], SourceMappingOriginal, () => [["line", option_type(int32_type)], ["column", option_type(int32_type)], ["Source", option_type(string_type)], ["Name", option_type(string_type)]]);
}

export class SourceChunk extends Union {
    constructor(tag, fields) {
        super();
        this.tag = tag;
        this.fields = fields;
    }
    cases() {
        return ["ChunkS", "ChunkArrS", "ChunkArrSN"];
    }
}

export function SourceChunk_$reflection() {
    return union_type("SourceMapSharp.SourceChunk", [], SourceChunk, () => [[["Item", string_type]], [["Item", array_type(string_type)]], [["Item", array_type(SourceNode_$reflection())]]]);
}

export class SourceNodeChild extends Union {
    constructor(tag, fields) {
        super();
        this.tag = tag;
        this.fields = fields;
    }
    cases() {
        return ["S", "SN"];
    }
    toString() {
        const this$ = this;
        return (this$.tag === 1) ? toString(this$.fields[0]) : this$.fields[0];
    }
}

export function SourceNodeChild_$reflection() {
    return union_type("SourceMapSharp.SourceNodeChild", [], SourceNodeChild, () => [[["Item", string_type]], [["Item", SourceNode_$reflection()]]]);
}

export class SourceNode {
    constructor(_line, _column, _source, _chunks, _name) {
        let arr, array;
        this.this = (new FSharpRef(defaultOf()));
        const this$ = this.this;
        this.this.contents = this;
        this._children = ((arr = [], (_chunks == null) ? arr : (((array = _chunks, array.forEach((chunk) => {
            SourceNode__addChunkToArray_4619EC48(this, chunk, arr);
        })), arr))));
        this["line@"] = _line;
        this["column@"] = _column;
        this["source@"] = _source;
        this["name@"] = _name;
        this["sourcesContents@"] = (new Map([]));
        this["init@36"] = 1;
    }
    toString() {
        const _ = this;
        const sb = StringBuilder_$ctor();
        SourceNode__Walk_Z4E9331C3(_.this.contents, (s, _arg) => {
            StringBuilder__Append_Z721C83C5(sb, s);
        });
        return toString(sb);
    }
}

export function SourceNode_$reflection() {
    return class_type("SourceMapSharp.SourceNode", void 0, SourceNode);
}

export function SourceNode_$ctor_Z6BDA1CA4(_line, _column, _source, _chunks, _name) {
    return new SourceNode(_line, _column, _source, _chunks, _name);
}

export function SourceNode__get_line(__) {
    return __["line@"];
}

export function SourceNode__get_column(__) {
    return __["column@"];
}

export function SourceNode__get_source(__) {
    return __["source@"];
}

export function SourceNode__get_name(__) {
    return __["name@"];
}

export function SourceNode__get_children(this$) {
    return this$._children;
}

export function SourceNode__set_children_Z2CD7AF84(this$, value) {
    this$._children = value;
}

export function SourceNode__get_sourcesContents(__) {
    return __["sourcesContents@"];
}

export function SourceNode__Add_331105B4(_, chunk) {
    SourceNode__addChunkToArray_4619EC48(_, chunk, SourceNode__get_children(_.this.contents));
    return _.this.contents;
}

export function SourceNode__Prepend_331105B4(_, chunk) {
    switch (chunk.tag) {
        case 1: {
            insertRangeInPlace(0, map_1((Item) => (new SourceNodeChild(0, [Item])), chunk.fields[0]), SourceNode__get_children(_.this.contents));
            break;
        }
        case 2: {
            insertRangeInPlace(0, map_1((Item_1) => (new SourceNodeChild(1, [Item_1])), chunk.fields[0]), SourceNode__get_children(_.this.contents));
            break;
        }
        default:
            SourceNode__get_children(_.this.contents).splice(0, 0, new SourceNodeChild(0, [chunk.fields[0]]));
    }
    return _.this.contents;
}

export function SourceNode__Prepend_Z66BD4C1(_, chunks) {
    for (let i = chunks.length - 1; i >= 0; i--) {
        SourceNode__Prepend_331105B4(_.this.contents, chunks[i]);
    }
    return _.this.contents;
}

export function SourceNode__Walk_Z4E9331C3(_, fn) {
    let enumerator = getEnumerator(SourceNode__get_children(_.this.contents));
    try {
        while (enumerator["System.Collections.IEnumerator.MoveNext"]()) {
            const chunk = enumerator["System.Collections.Generic.IEnumerator`1.get_Current"]();
            if (chunk.tag === 1) {
                SourceNode__Walk_Z4E9331C3(chunk.fields[0], fn);
            }
            else {
                fn(chunk.fields[0], new SourceMappingOriginal(SourceNode__get_line(_.this.contents), SourceNode__get_column(_.this.contents), SourceNode__get_source(_.this.contents), SourceNode__get_name(_.this.contents)));
            }
        }
    }
    finally {
        disposeSafe(enumerator);
    }
}

export function SourceNode__Join_Z721C83C5(_, sep) {
    if (SourceNode__get_children(_.this.contents).length > 0) {
        const newChildren = [];
        const len = SourceNode__get_children(_.this.contents).length | 0;
        for (let i = 0; i <= (len - 2); i++) {
            void (newChildren.push(SourceNode__get_children(_.this.contents)[i]));
            void (newChildren.push(new SourceNodeChild(0, [sep])));
        }
        void (newChildren.push(SourceNode__get_children(_.this.contents)[len - 1]));
        SourceNode__set_children_Z2CD7AF84(_.this.contents, newChildren);
    }
    return _.this.contents;
}

export function SourceNode__ReplaceRight_Z384F8060(__mut, pattern_mut, replacement_mut) {
    SourceNode__ReplaceRight_Z384F8060:
    while (true) {
        const _ = __mut, pattern = pattern_mut, replacement = replacement_mut;
        const lastChild = tryLast(SourceNode__get_children(_.this.contents));
        if (lastChild == null) {
            void (SourceNode__get_children(_.this.contents).push(new SourceNodeChild(0, [replace("", pattern, replacement)])));
            return _.this.contents;
        }
        else if (lastChild.tag === 0) {
            const str = lastChild.fields[0];
            SourceNode__get_children(_.this.contents)[SourceNode__get_children(_.this.contents).length - 1] = (new SourceNodeChild(0, [replace(str, pattern, replacement)]));
            return _.this.contents;
        }
        else {
            const sn = lastChild.fields[0];
            __mut = sn;
            pattern_mut = pattern;
            replacement_mut = replacement;
            continue SourceNode__ReplaceRight_Z384F8060;
        }
        break;
    }
}

export function SourceNode__SetSourceContent_Z384F8060(_, file, content) {
    SourceNode__get_sourcesContents(_.this.contents).set(file, content);
}

export function SourceNode__WalkSourceContents_6C2984E5(_, fn) {
    let enumerator = getEnumerator(SourceNode__get_children(_.this.contents));
    try {
        while (enumerator["System.Collections.IEnumerator.MoveNext"]()) {
            const child = enumerator["System.Collections.Generic.IEnumerator`1.get_Current"]();
            if (child.tag === 1) {
                SourceNode__WalkSourceContents_6C2984E5(child.fields[0], fn);
            }
        }
    }
    finally {
        disposeSafe(enumerator);
    }
    iterate((kvp) => {
        fn([kvp[0], kvp[1]]);
    }, SourceNode__get_sourcesContents(_.this.contents));
}

export function SourceNode__ToStringWithSourceMap_Z257FAB7(_, skipValidation, file, sourceRoot) {
    let z_3, y_3, y_2, z_2, x_3, x_2, z_1, x_1, y_1, x, y, z;
    const map = (skipValidation == null) ? ((file == null) ? ((sourceRoot == null) ? SourceMapGenerator_$ctor_Z257FAB7() : ((z_3 = sourceRoot, SourceMapGenerator_$ctor_Z257FAB7(void 0, void 0, z_3)))) : ((sourceRoot == null) ? ((y_3 = file, SourceMapGenerator_$ctor_Z257FAB7(void 0, y_3))) : ((y_2 = file, (z_2 = sourceRoot, SourceMapGenerator_$ctor_Z257FAB7(void 0, y_2, z_2)))))) : ((file == null) ? ((sourceRoot == null) ? ((x_3 = skipValidation, SourceMapGenerator_$ctor_Z257FAB7(x_3))) : ((x_2 = skipValidation, (z_1 = sourceRoot, SourceMapGenerator_$ctor_Z257FAB7(x_2, void 0, z_1))))) : ((sourceRoot == null) ? ((x_1 = skipValidation, (y_1 = file, SourceMapGenerator_$ctor_Z257FAB7(x_1, y_1)))) : ((x = skipValidation, (y = file, (z = sourceRoot, SourceMapGenerator_$ctor_Z257FAB7(x, y, z)))))));
    let generatedCode = StringBuilder_$ctor();
    let generatedLine = 1;
    let generatedColumn = 0;
    let sourceMappingActive = false;
    let lastOriginalSource = void 0;
    let lastOriginalLine = void 0;
    let lastOriginalColumn = void 0;
    let lastOriginalName = void 0;
    SourceNode__Walk_Z4E9331C3(_.this.contents, (chunk, original) => {
        StringBuilder__Append_Z721C83C5(generatedCode, chunk);
        if (((original.Source != null) && (original.line != null)) && (original.column != null)) {
            if (((!equals(lastOriginalSource, original.Source) ? true : !equals(lastOriginalLine, original.line)) ? true : !equals(lastOriginalColumn, original.column)) ? true : !equals(lastOriginalName, original.Name)) {
                const _generated = new MappingIndex(generatedLine, generatedColumn);
                const _original = new MappingIndex(value_1(original.line), value_1(original.column));
                const matchValue_1 = original.Name;
                if (matchValue_1 == null) {
                    SourceMapGenerator__AddMapping_Z1908099D(map, _generated, _original, value_1(original.Source));
                }
                else {
                    const n = matchValue_1;
                    SourceMapGenerator__AddMapping_Z1908099D(map, _generated, _original, value_1(original.Source), n);
                }
            }
            lastOriginalSource = original.Source;
            lastOriginalLine = original.line;
            lastOriginalColumn = original.column;
            lastOriginalName = original.Name;
            sourceMappingActive = true;
        }
        else if (sourceMappingActive) {
            SourceMapGenerator__AddMapping_Z1908099D(map, new MappingIndex(generatedLine, generatedColumn));
            lastOriginalSource = void 0;
            sourceMappingActive = false;
        }
        const enumerator = getEnumerator(toList(rangeDouble(0, 1, chunk.length - 1)));
        try {
            while (enumerator["System.Collections.IEnumerator.MoveNext"]()) {
                const idx = enumerator["System.Collections.Generic.IEnumerator`1.get_Current"]() | 0;
                if (chunk[idx].charCodeAt(0) === SourceNodeModule_NEWLINE_CODE) {
                    generatedLine = ((generatedLine + 1) | 0);
                    generatedColumn = 0;
                    if ((idx + 1) === chunk.length) {
                        lastOriginalSource = void 0;
                        sourceMappingActive = false;
                    }
                    else if (sourceMappingActive) {
                        const _generated_2 = new MappingIndex(generatedLine, generatedColumn);
                        const _original_1 = new MappingIndex(value_1(original.line), value_1(original.column));
                        const matchValue_2 = original.Source;
                        const matchValue_3 = original.Name;
                        if (matchValue_2 == null) {
                            if (matchValue_3 == null) {
                                SourceMapGenerator__AddMapping_Z1908099D(map, _generated_2, _original_1);
                            }
                            else {
                                const name_1 = matchValue_3;
                                SourceMapGenerator__AddMapping_Z1908099D(map, _generated_2, _original_1, void 0, name_1);
                            }
                        }
                        else if (matchValue_3 == null) {
                            const src_1 = matchValue_2;
                            SourceMapGenerator__AddMapping_Z1908099D(map, _generated_2, _original_1, src_1);
                        }
                        else {
                            const name = matchValue_3;
                            const src = matchValue_2;
                            SourceMapGenerator__AddMapping_Z1908099D(map, _generated_2, _original_1, src, name);
                        }
                    }
                }
                else {
                    generatedColumn = ((generatedColumn + 1) | 0);
                }
            }
        }
        finally {
            disposeSafe(enumerator);
        }
    });
    SourceNode__WalkSourceContents_6C2984E5(_.this.contents, (tupledArg) => {
        SourceMapGenerator__SetSourceContent_27AED5E3(map, tupledArg[0], tupledArg[1]);
    });
    return [toString(generatedCode), map];
}

export function SourceNode__addChunkToArray_4619EC48(this$, chunk, arr) {
    switch (chunk.tag) {
        case 1: {
            chunk.fields[0].forEach((x) => {
                void (arr.push(new SourceNodeChild(0, [x])));
            });
            break;
        }
        case 2: {
            chunk.fields[0].forEach((x_1) => {
                void (arr.push(new SourceNodeChild(1, [x_1])));
            });
            break;
        }
        default:
            void (arr.push(new SourceNodeChild(0, [chunk.fields[0]])));
    }
}

