import { Record } from "../../fable-library.4.5.0/Types.js";
import { bool_type, class_type, string_type, option_type, record_type, int32_type } from "../../fable-library.4.5.0/Reflection.js";
import { compare, equals } from "../../fable-library.4.5.0/Util.js";
import { isNullOrWhiteSpace, replace, substring, trimStart, trimEnd } from "../../fable-library.4.5.0/String.js";

export class MappingIndex extends Record {
    constructor(line, column) {
        super();
        this.line = (line | 0);
        this.column = (column | 0);
    }
}

export function MappingIndex_$reflection() {
    return record_type("SourceMapSharp.Util.MappingIndex", [], MappingIndex, () => [["line", int32_type], ["column", int32_type]]);
}

export class Mapping extends Record {
    constructor(Generated, Original, Source, Name) {
        super();
        this.Generated = Generated;
        this.Original = Original;
        this.Source = Source;
        this.Name = Name;
    }
}

export function Mapping_$reflection() {
    return record_type("SourceMapSharp.Util.Mapping", [], Mapping, () => [["Generated", MappingIndex_$reflection()], ["Original", option_type(MappingIndex_$reflection())], ["Source", option_type(string_type)], ["Name", option_type(string_type)]]);
}

export class SourceGeneratorJSON extends Record {
    constructor(version, sources, names, mappings, file, sourcesContent, sourceRoot) {
        super();
        this.version = (version | 0);
        this.sources = sources;
        this.names = names;
        this.mappings = mappings;
        this.file = file;
        this.sourcesContent = sourcesContent;
        this.sourceRoot = sourceRoot;
    }
}

export function SourceGeneratorJSON_$reflection() {
    return record_type("SourceMapSharp.Util.SourceGeneratorJSON", [], SourceGeneratorJSON, () => [["version", int32_type], ["sources", class_type("System.Collections.Generic.IEnumerable`1", [string_type])], ["names", class_type("System.Collections.Generic.IEnumerable`1", [string_type])], ["mappings", string_type], ["file", option_type(string_type)], ["sourcesContent", option_type(class_type("System.Collections.Generic.IEnumerable`1", [option_type(string_type)]))], ["sourceRoot", option_type(string_type)]]);
}

export class RawSection extends Record {
    constructor(offset, map) {
        super();
        this.offset = offset;
        this.map = map;
    }
}

export function RawSection_$reflection() {
    return record_type("SourceMapSharp.Util.RawSection", [], RawSection, () => [["offset", MappingIndex_$reflection()], ["map", SourceGeneratorJSON_$reflection()]]);
}

export class RawIndexMap extends Record {
    constructor(version, file, sourceRoot, skipValidation, sections) {
        super();
        this.version = (version | 0);
        this.file = file;
        this.sourceRoot = sourceRoot;
        this.skipValidation = skipValidation;
        this.sections = sections;
    }
}

export function RawIndexMap_$reflection() {
    return record_type("SourceMapSharp.Util.RawIndexMap", [], RawIndexMap, () => [["version", int32_type], ["file", option_type(string_type)], ["sourceRoot", option_type(string_type)], ["skipValidation", option_type(bool_type)], ["sections", class_type("System.Collections.Generic.IEnumerable`1", [RawSection_$reflection()])]]);
}

export function strcmp(s1, s2) {
    if ((s1 != null) && (s2 != null)) {
        if (equals(s1, s2)) {
            return 0;
        }
        else if (compare(s1, s2) > 0) {
            return 1;
        }
        else {
            return -1;
        }
    }
    else if ((s1 == null) && (s2 == null)) {
        return 0;
    }
    else if (s1 == null) {
        return 1;
    }
    else if (s2 == null) {
        return -1;
    }
    else {
        return -1;
    }
}

export function compareByGeneratedPositionsInflated(mappingA, mappingB) {
    let cmp = mappingA.Generated.line - mappingB.Generated.line;
    if (cmp !== 0) {
        return cmp | 0;
    }
    else {
        cmp = ((mappingA.Generated.column - mappingB.Generated.column) | 0);
        if (cmp !== 0) {
            return cmp | 0;
        }
        else {
            cmp = (strcmp(mappingA.Source, mappingB.Source) | 0);
            if (cmp !== 0) {
                return cmp | 0;
            }
            else {
                const matchValue = mappingA.Original;
                const matchValue_1 = mappingB.Original;
                let matchResult, mapAOriginal, mapBOriginal;
                if (matchValue != null) {
                    if (matchValue_1 != null) {
                        matchResult = 0;
                        mapAOriginal = matchValue;
                        mapBOriginal = matchValue_1;
                    }
                    else {
                        matchResult = 1;
                    }
                }
                else {
                    matchResult = 1;
                }
                switch (matchResult) {
                    case 0: {
                        cmp = ((mapAOriginal.line - mapBOriginal.line) | 0);
                        if (cmp !== 0) {
                            return cmp | 0;
                        }
                        else {
                            cmp = ((mapAOriginal.column - mapBOriginal.column) | 0);
                            if (cmp !== 0) {
                                return cmp | 0;
                            }
                            else {
                                return strcmp(mappingA.Name, mappingB.Name) | 0;
                            }
                        }
                    }
                    default:
                        return strcmp(mappingA.Name, mappingB.Name) | 0;
                }
            }
        }
    }
}

function Combine(path1, path2) {
    return ((path1.length === 0) ? path1 : (trimEnd(path1, "\\", "/") + "/")) + trimStart(path2, "\\", "/");
}

function ChangeExtension(path, ext) {
    const i = path.lastIndexOf(".") | 0;
    if (i < 0) {
        return path;
    }
    else {
        return substring(path, 0, i) + ext;
    }
}

function GetExtension(path) {
    const i = path.lastIndexOf(".") | 0;
    if (i < 0) {
        return "";
    }
    else {
        return substring(path, i);
    }
}

function normalizePath(path) {
    return replace(path, "\\", "/");
}

function isRelativePath(path) {
    const len = path.length | 0;
    if (len === 0) {
        return false;
    }
    else if (path[0] === ".") {
        if (len === 1) {
            return true;
        }
        else {
            const matchValue = path[1];
            switch (matchValue) {
                case ".":
                case "/":
                case "\\":
                    return true;
                default:
                    return false;
            }
        }
    }
    else {
        return false;
    }
}

function getRelativeFileOrDirPath(fromIsDir, fromFullPath, toIsDir, toFullPath) {
    let path1, path2, c, d, builder;
    const addDummyFile = (isDir, path) => {
        if (isDir) {
            return Combine(path, "__DUMMY-FILE__.txt");
        }
        else {
            return path;
        }
    };
    const fromFullPath_1 = normalizePath(fromFullPath);
    const toFullPath_1 = normalizePath(toFullPath);
    const matchValue = replace((path1 = addDummyFile(fromIsDir, fromFullPath_1), (path2 = addDummyFile(toIsDir, toFullPath_1), (c = 0, (d = -1, ((() => {
        while (((c < path1.length) && (c < path2.length)) && (path1[c] === path2[c])) {
            if (path1[c] === "/") {
                d = (c | 0);
            }
            c = ((c + 1) | 0);
        }
    })(), (c === 0) ? path2 : (((c === path1.length) && (c === path2.length)) ? "" : ((builder = "", ((() => {
        while (c < path1.length) {
            if (path1[c] === "/") {
                builder = (builder + "../");
            }
            c = ((c + 1) | 0);
        }
    })(), ((builder.length === 0) && ((path2.length - 1) === d)) ? "./" : (builder + substring(path2, d + 1))))))))))), "__DUMMY-FILE__.txt", "");
    if (matchValue === "") {
        return ".";
    }
    else {
        return matchValue;
    }
}

export function getRelativePath(fromFullPath, toFullPath) {
    const isDir = (arg) => isNullOrWhiteSpace(GetExtension(arg));
    return getRelativeFileOrDirPath(isDir(fromFullPath), fromFullPath, isDir(toFullPath), toFullPath);
}

export function computeSourceUrl(sourceRoot, sourceUrl, sourceMapUrl) {
    return sourceUrl;
}

export class MappingComparer {
    constructor() {
    }
    Compare(a, b) {
        return compareByGeneratedPositionsInflated(a, b);
    }
}

export function MappingComparer_$reflection() {
    return class_type("SourceMapSharp.Util.MappingComparer", void 0, MappingComparer);
}

export function MappingComparer_$ctor() {
    return new MappingComparer();
}

